@@ -3,8 +3,8 @@
  *
  *   (C) 2003 Dominic Morris
  *
- *   $Id: gui.c,v 1.13 2004/02/22 19:20:54 dom Exp $
- *   $Date: 2004/02/22 19:20:54 $
+ *   $Id: gui.c,v 1.25 2004/05/07 22:50:13 dom Exp $
+ *   $Date: 2004/05/07 22:50:13 $
  *
  *
  *   Handles the GUI...
@@ -18,18 +18,16 @@
 
 
 struct _gui {
-    dongle_t     *dongle;
-    struct event  event;
-    int           fd;
-    int           state;
-    uint8_t       send_update;      /* Send an update out when necessary */
-    render_t     *render;           /* Rendering surface */
-    void         *app;              /* Controlling application */
-    char          inbuf[1024];
-    int           inbufstart;
+    application_t  application;      /* Controlling application spec */
+    dongle_t      *dongle;
+    struct event   event;
+    int            fd;
+    int            state;
+    uint8_t        send_update;      /* Send an update out when necessary */
+    char           inbuf[1024];
+    int            inbufstart;
 };
 
-
 static void       gui_accept(int fd, short event, void *arg);
 static void       gui_read(int fd, short event, void *arg);
 static void       rfb_send_data(gui_t *gui);
@@ -46,20 +44,31 @@ static int          listen_sock;
 static mvprfbmsg_t  *rdc_msgs[RDC_MESSAGE_MAX];
 static mvprfbmsg_t  *rdc_msgs_22012[RDC_MESSAGE_MAX];
 
+#define DEBUG_TIMESTAMP debug_timestamp()
+
+static char buf[30];
+static const char * debug_timestamp()
+{
+   struct timeval tv;
+   gettimeofday(&tv, NULL);
+   snprintf(buf,30,"[TS: %ld.%03ld]", tv.tv_sec, (tv.tv_usec+500)/1000);
+   return buf;
+}
 
 
 int gui_init()
 {
 
-    if ( (listen_sock = tcp_listen(NULL,initparams.gui_port,0) ) == -1 ) {
-        return -1;      
+    if ( (listen_sock = tcp_listen(NULL,initparams->gui_port,0) ) == -1 ) {
+        Dprintf(ERROR,"Cannot listen to port %d\n",initparams->gui_port);
+        return -1;
     }
 
     event_set(&listen_event,listen_sock,EV_READ,gui_accept,&listen_event);
 
     event_add(&listen_event,NULL);
 
-    Dprintf(DEBUG,"Listening on port %d for GUI connection\n",initparams.gui_port);
+    Dprintf(INFO,"Listening on port %d for GUI connection\n",initparams->gui_port);
 
     /* Set up some structures */
     rdc_msgs[RDC_PLAY] = &rfb_media_play;
@@ -74,8 +83,11 @@ int gui_init()
     rdc_msgs[RDC_SETTINGS] = &rfb_media_settings;
     rdc_msgs[RDC_DISPLAY_ON] = &rfb_media_displayon;
     rdc_msgs[RDC_DISPLAY_OFF] = &rfb_media_displayoff;
+    rdc_msgs[RDC_SKIP] = &rfb_media_stop; // not very nice to user but safe
+    rdc_msgs[RDC_BACK] = &rfb_media_stop; // not very nice to user but safe
+    rdc_msgs[RDC_SEEK_PERCENT] = &rfb_media_stop; // not very nice to user but safe
     rdc_msgs[RFB_PONG]  = &rfb_ping;
-         
+
     rdc_msgs_22012[RDC_PLAY] = &rfb_media_play_22012;
     rdc_msgs_22012[RDC_PAUSE] = &rfb_media_pause_22012;
     rdc_msgs_22012[RDC_STOP] = &rfb_media_stop_22012;
@@ -88,8 +100,11 @@ int gui_init()
     rdc_msgs_22012[RDC_SETTINGS] = &rfb_media_settings_22012;
     rdc_msgs_22012[RDC_DISPLAY_ON] = &rfb_media_displayon_22012;
     rdc_msgs_22012[RDC_DISPLAY_OFF] = &rfb_media_displayoff_22012;
+    rdc_msgs_22012[RDC_SKIP] = &rfb_media_skip_22012;
+    rdc_msgs_22012[RDC_BACK] = &rfb_media_back_22012;
+    rdc_msgs_22012[RDC_SEEK_PERCENT] = &rfb_media_percent_22012;
     rdc_msgs_22012[RFB_PONG]  = &rfb_ping;
-    
+
 
 
     return 0;
@@ -105,22 +120,21 @@ static gui_t *new_gui(dongle_t *dongle)
 {
     gui_t  *gui;
 
-    gui = malloc(sizeof(*gui));
-    dongle->gui = gui;
+    gui = (gui_t*)calloc(1,sizeof(*gui));
+    if ( ( initparams->open(dongle,&gui->application) ) < 0 ) {
+        free(gui);
+        return NULL;
+    }
 
+    /* Now set up the rest of the parameters */
+    dongle->gui = gui;
     gui->dongle = dongle;
     gui->state  = VNC_VERSION;
     gui->inbufstart = 0;
     gui->send_update = TRUE;
-    gui->render = new_render(initparams.gui_width,initparams.gui_depth);
-    if ( initparams.new_app ) {
-        gui->app    = initparams.new_app(dongle,gui->render); 
-    } else {
-        /* No app...lets try and do the best we can.. */
-        gui->app = NULL;
-    }
-
 
+  
+   
     return gui;
 }
 
@@ -129,16 +143,13 @@ void delete_gui(gui_t *gui)
     event_del(&gui->event);
     close(gui->fd);
     shutdown(gui->fd,SHUT_RDWR);
-    delete_render(gui->render);
-    if ( initparams.delete_app ) {
-        initparams.delete_app(gui->app);         
-    }
+    gui->application.close(&gui->application);
     free(gui);
 }
 
 static void gui_accept(int fd, short event, void *arg)
 {
-    struct event  *ev = arg;
+    struct event  *ev = (struct event*)arg;
     dongle_t      *dongle;
     gui_t         *gui;
     char          *hostname;
@@ -151,21 +162,28 @@ static void gui_accept(int fd, short event, void *arg)
     if ( ( cfd = tcp_accept(fd,&hostname) ) == -1 ) {
         return;
     }
-    Dprintf(DEBUG,"Accepted GUI connection from host %s\n",hostname);
+    Dprintf(INFO,"Accepted GUI connection from host %s\n",hostname);
     dongle = dongle_return(hostname);
 
     if ( dongle->gui ) {
         dongle_close(dongle);
         dongle = dongle_return(hostname);
     }
-            
 
-    gui = new_gui(dongle);
+  
+    /* Create a new application, if we fail, then abort */
+    if ( ( gui = new_gui(dongle) ) == NULL ) {
+        dongle_close(dongle);
+        close(cfd);
+        shutdown(cfd,SHUT_RDWR);
+        return;
+    }
 
     gui->fd = cfd;
 
     event_set(&gui->event,cfd,EV_READ,gui_read,gui);
     event_add(&gui->event,NULL);
+
     rfb_send_data(gui);      /* Send out the protocol info */
 }
 
@@ -173,7 +191,7 @@ static void gui_accept(int fd, short event, void *arg)
 
 static void gui_read(int fd, short event, void *arg)
 {
-    gui_t       *gui = arg;
+    gui_t       *gui = (gui_t*)arg;
     int          r,diff = 0 ;
 
     /* Reschedule event */
@@ -186,7 +204,7 @@ static void gui_read(int fd, short event, void *arg)
         dongle_close(gui->dongle);
         return;
     }
-  
+
     /* We only expect to get one message a time */
 
     gui->inbufstart += r;
@@ -200,7 +218,7 @@ static void gui_read(int fd, short event, void *arg)
                 diff = 12;
             } else {
                 diff = 0;
-            }           
+            }
             break;
         case VNC_AUTH:
         case VNC_AUTH2:
@@ -208,9 +226,10 @@ static void gui_read(int fd, short event, void *arg)
             gui->state = VNC_INIT;
             break;
         case VNC_RUNNING:
-            diff = rfb_running(gui,gui->inbuf,gui->inbufstart);
+            diff = rfb_running(gui,(unsigned char*)gui->inbuf,gui->inbufstart);
             break;
         default:
+            break;
         }
         if ( diff <= 0 ) {
             break;
@@ -222,30 +241,35 @@ static void gui_read(int fd, short event, void *arg)
             } else {
                 gui->inbufstart = 0;
             }
-        }          
+        }
+
+        /* Send out pending messages */
+        if ( diff != -1 ) {
+            rfb_send_data(gui);
+        }
+
+        Dprintf(DEBUG, "%s finished processing.\n", DEBUG_TIMESTAMP);
     }
 
-    /* Send out pending messages */
-    if ( diff != -1 ) {
-        rfb_send_data(gui);
-    }
 
 
 }
 
-/** \brief Handle incoming RFB messages from the client
+/** \internal
+ *  \brief Handle incoming RFB messages from the client
  *
  * \param gui Handle to the gui channel
  * \param buf Incoming message buffer
  * \param len Length of the incoming message buffer
  *
  * \return Number of bytes processed
- */ 
+ */
 static int rfb_running(gui_t *gui, unsigned char *buf, int len)
 {
     uint8_t  *ptr = buf + 2;
     int       diff = 0;
 
+    Dprintf(DEBUG, "%s: processing %d\n", DEBUG_TIMESTAMP, buf[0]);
     switch ( buf[0] ) {
     case RFB_SET_PIXEL_FORMAT:
         if ( len >= 20 ) {
@@ -259,7 +283,7 @@ static int rfb_running(gui_t *gui, unsigned char *buf, int len)
             uint8_t   redshift;
             uint8_t   greenshift;
             uint8_t   blueshift;
-        
+
             ptr += 2;  /* Skip padding */
 
             bpp = *ptr++;
@@ -278,7 +302,7 @@ static int rfb_running(gui_t *gui, unsigned char *buf, int len)
                    redmax,greenmax,bluemax,
                    redshift,greenshift,blueshift);
             diff = 20;
-        }       
+        }
         break;
     case RFB_FIX_COLORMAP_ENTRIES:
         /* Shouldn't get here */
@@ -288,7 +312,7 @@ static int rfb_running(gui_t *gui, unsigned char *buf, int len)
             uint16_t  num;
             uint32_t  type;
             int       i;
-            
+
             BUF_TO_INT16(num,ptr);
             Dprintf(DEBUG,"Client supports %d encodings: ",num);
             if ( len >= 4 + ( num * 4 ) ) {
@@ -328,7 +352,7 @@ static int rfb_running(gui_t *gui, unsigned char *buf, int len)
             int        code;
 
             ptr = buf + 4;
-            BUF_TO_INT32(key,ptr);            
+            BUF_TO_INT32(key,ptr);
 
             code = key_code(key);
 
@@ -343,11 +367,9 @@ static int rfb_running(gui_t *gui, unsigned char *buf, int len)
                 }
 
                 /* Just pass it back to a higher level */
-                if ( initparams.app_keypress ) {
-                    initparams.app_keypress(gui->app,code);              
-                }
-            }            
-        
+                gui->application.keypress(&gui->application,code);
+            }
+
             diff = 8;
         }
         break;
@@ -366,29 +388,29 @@ static int rfb_running(gui_t *gui, unsigned char *buf, int len)
 
             Dprintf(DEBUG,"Received client cuttext of length %d\n",tlen);
 
-            if ( len >= 8 + tlen ) {
+            if ( (uint32_t)len >= 8 + tlen ) {
                 diff = 8 + tlen;
             }
         }
         break;
-    case RFB_RDC_ACK:     
+    case RFB_RDC_ACK:
         if ( len >= 10 ) {
-            if ( initparams.dongle_version < 21365 ) {
+            if ( initparams->dongle_version < 21365 ) {
                 diff = 10;
             } else {
                 diff = 34;
             }
 
-            switch ( buf[1] ) {                          
+            switch ( buf[1] ) {
             case RDC_SETTINGS:       /* This really is the only one we need to catch */
                 if ( buf[7] != 0x00 ) {
-                    if ( diff + buf[7] >= len ) {                   
+                    if ( diff + buf[7] >= len ) {
 
                         /* Save those settings */
                         gui->dongle->tvmode = buf[diff+1];
                         /* output mode seems to be supplied with -1 - quite why I've no idea! */
                         gui->dongle->videooutput = buf[diff+2];
-                        if ( initparams.dongle_version < 21330 ) {
+                        if ( initparams->dongle_version < 21330 ) {
                             gui->dongle->videooutput--;
                         }
                         gui->dongle->flickermode = buf[diff + 3];
@@ -401,15 +423,13 @@ static int rfb_running(gui_t *gui, unsigned char *buf, int len)
                         diff += buf[7];
                     } else {
                         diff = 0;  /* Not had all the data yet */
-                    }                   
-                } 
+                    }
+                }
                 break;
             default:
                 /* Kick ack events up to a higher level so we can do multiple plays */
-                if ( initparams.app_handle_ack ) {
-                    initparams.app_handle_ack(gui->app,buf[1],buf,len); 
-                }
-              
+                gui->application.ack(&gui->application,buf[1],buf,len);
+
                 break;
             }
         }
@@ -422,7 +442,7 @@ static int rfb_running(gui_t *gui, unsigned char *buf, int len)
         break;
     default:
         dump_hex(INFO,buf,len);
-    
+
     }
 
     return diff;
@@ -438,7 +458,7 @@ static void rfb_send_data(gui_t *gui)
 
     switch ( gui->state ) {
     case VNC_VERSION:
-        len = sprintf(buf,"RFB %03d.%03d\012",RFB_MAJOR,RFB_MINOR);
+        len = sprintf((char*)buf,"RFB %03d.%03d\012",RFB_MAJOR,RFB_MINOR);
         write(gui->fd,buf,len);
         break;
     case VNC_AUTH:
@@ -450,8 +470,8 @@ static void rfb_send_data(gui_t *gui)
         break;
     case VNC_INIT:
         ptr = buf;
-        INT16_TO_BUF(initparams.screen_width,ptr);
-        INT16_TO_BUF(initparams.screen_depth,ptr);
+        INT16_TO_BUF(gui->application.screen_width,ptr);
+        INT16_TO_BUF(gui->application.screen_depth,ptr);
         *ptr++ = 24;    /* bpp */
         *ptr++ = 24;    /* depth */
         *ptr++ = 0;     /* big endian */
@@ -467,27 +487,28 @@ static void rfb_send_data(gui_t *gui)
         *ptr++ = 0;     /* padding */
         len = strlen("rfbwindows");
         INT32_TO_BUF(len,ptr);
-        strcpy(ptr,"rfbwindows");
+        strcpy((char*)ptr,"rfbwindows");
         ptr += len;
         write(gui->fd,buf,ptr - buf);
         gui_send_settings(gui,RDC_SETTINGS_GET);
         gui->state = VNC_RUNNING;
-        break;    
-    case VNC_RUNNING:        
+        break;
+    case VNC_RUNNING:
         if ( gui->send_update ) {
             rfb_send_screen(gui);
-        }       
+        }
         break;
-    }        
+    }
 }
 
-/** \brief Send a screen update to the client
+/** \internal
+ *  \brief Send a screen update to the client
  *
  * \param gui Handle to the gui channel
  *
- * We have to translate our rgb map into yuvy, compress it and then 
+ * We have to translate our rgb map into yuvy, compress it and then
  * send it down the rfb channel
- */ 
+ */
 static void rfb_send_screen(gui_t *gui)
 {
     uint8_t        buf[100];
@@ -497,24 +518,22 @@ static void rfb_send_screen(gui_t *gui)
     int            yuvsize;
     int            gz;
 
-    surface = render_rgb2yvuy(gui->render);
-
-    if ( surface == NULL ) {
+    if ( ( surface = gui->application.get_yvuy_surface(&gui->application,&yuvsize) ) == NULL ) {
         return;
     }
-    Dprintf(DEBUG,"Sending update\n");
+
     gui->send_update = FALSE;
-    yuvsize = render_get_yuvsize(gui->render);
-    comp = calloc(yuvsize,1);  
+    Dprintf(DEBUG,"%s Sending update size %d\n", DEBUG_TIMESTAMP,yuvsize);
+    comp = (uint8_t*)calloc(yuvsize,1);
 
     *ptr++ = RFB_FB_UPDATE;
     *ptr++ = 0xCC;            /* Pad */
     INT16_TO_BUF(1,ptr);      /* Number rectangles */
-    INT16_TO_BUF((initparams.screen_width - render_get_width(gui->render))/ 2,ptr);      /* x pos */
-    INT16_TO_BUF((initparams.screen_depth - render_get_depth(gui->render))/ 2,ptr);      /* y pos */
+    INT16_TO_BUF((gui->application.screen_width - gui->application.gui_width ) / 2,ptr);
+    INT16_TO_BUF((gui->application.screen_depth - gui->application.gui_depth ) / 2,ptr);
 
-    INT16_TO_BUF(render_get_width(gui->render),ptr);      /* width */
-    INT16_TO_BUF(render_get_depth(gui->render),ptr);      /* depth */
+    INT16_TO_BUF(gui->application.gui_width,ptr);      /* width */
+    INT16_TO_BUF(gui->application.gui_depth,ptr);      /* depth */
     INT32_TO_BUF(7,ptr);      /* The encoding type used by HCW... */
 
 
@@ -522,22 +541,24 @@ static void rfb_send_screen(gui_t *gui)
     INT32_TO_BUF(gz,ptr);
     INT16_TO_BUF(0,ptr);        /* colour format, 0 = yuv */
     INT16_TO_BUF(2,ptr);        /* bytes per pixel */
-       
+
     write(gui->fd,buf,ptr - buf);
     write(gui->fd,comp,gz);
-        
+
     free(comp);
     free(surface);
+
 }
 
 #define CHECK_ERR(err, msg) { \
     if (err != Z_OK) { \
-        fprintf(stderr, "%s error: %d\n", msg, err); \
+        Dprintf(ERROR, "libz error: %s error: %d\n", msg, err); \
         exit(1); \
     } \
 }
 
-/** \brief deflate the yuvy buffer
+/** \internal
+ *  \brief deflate the yuvy buffer
  *
  *  \param uncompr The raw buffer
  *  \param uncomprLen Length of the raw buffer
@@ -546,7 +567,7 @@ static void rfb_send_screen(gui_t *gui)
  *
  *  Shamelessly stolen from a libz example
  */
-static int deflateit(Byte *uncompr, uLong uncomprLen, Byte *compr, uLong comprLen) 
+static int deflateit(Byte *uncompr, uLong uncomprLen, Byte *compr, uLong comprLen)
 {
     z_stream c_stream; /* compression stream */
     int err;
@@ -560,13 +581,13 @@ static int deflateit(Byte *uncompr, uLong uncomprLen, Byte *compr, uLong comprLe
 
     c_stream.next_out = compr;
     c_stream.avail_out = (uInt)comprLen;
-  
+
     c_stream.next_in = uncompr;
     c_stream.avail_in = (uInt)uncomprLen;
     err = deflate(&c_stream, Z_FINISH);
     // CHECK_ERR(err, "deflate");
-   
-   
+
+
 
     err = deflateEnd(&c_stream);
     CHECK_ERR(err, "deflateEnd");
@@ -575,9 +596,19 @@ static int deflateit(Byte *uncompr, uLong uncomprLen, Byte *compr, uLong comprLe
 }
 
 
+/** \internal
+ *  \brief Force a a screen update to the client
+ *
+ *  \param gui Handle to the gui channel 
+ */
+void gui_send_screen(gui_t *gui)
+{
+    rfb_send_screen(gui);
+}
 
 
-/** \brief Send a filename to the client down the RFB channel
+/** \internal
+ *  \brief Send a filename to the client down the RFB channel
  *
  * \param gui Handle to the gui channel
  * \param name Filename to be send (may well be http:// form as well)
@@ -585,14 +616,16 @@ static int deflateit(Byte *uncompr, uLong uncomprLen, Byte *compr, uLong comprLe
  * The rfb_media_filename message is unusual in that we need to nadger
  * it before the client will accept it.
  *
- */  
+ */
 void gui_send_play(gui_t *gui, char *name)
 {
     unsigned  char  buf[FILENAME_MAX + 34];
-    mvprfbmsg_t    *msg; 
+    mvprfbmsg_t    *msg;
     int             len = strlen(name);
 
-    if ( initparams.dongle_version < 21365 ) {
+    Dprintf(DEBUG, "%s Sending message %s\n", DEBUG_TIMESTAMP, name);
+
+    if ( initparams->dongle_version < 21365 ) {
         msg = &rfb_media_play;
     } else {
         msg = &rfb_media_play_22012;
@@ -602,12 +635,13 @@ void gui_send_play(gui_t *gui, char *name)
 
     buf[8] = len;        /* What happens here if path > 256?!?! */
 
-    strcpy(buf + msg->len,name);
+    strcpy((char*)(buf + msg->len),name);
 
-    write(gui->fd,buf,len + msg->len);    
+    write(gui->fd,buf,len + msg->len);
 }
 
-/** \brief Send a message to the client down the RFB channel
+/** \internal
+ *  \brief Send a message to the client down the RFB channel
  *
  * \param gui Handle to the gui channel
  * \param msgtype Message type to send
@@ -618,7 +652,9 @@ void gui_send_message(gui_t *gui, int msgtype)
     mvprfbmsg_t **msglist;
     mvprfbmsg_t  *msg;
 
-    if ( initparams.dongle_version < 21365 ) {
+    Dprintf(DEBUG, "%s Sending message %d\n", DEBUG_TIMESTAMP, msgtype);
+
+    if ( initparams->dongle_version < 21365 ) {
         msglist = rdc_msgs;
     } else {
         msglist = rdc_msgs_22012;
@@ -629,9 +665,29 @@ void gui_send_message(gui_t *gui, int msgtype)
     write(gui->fd,msg->msg,msg->len);
 }
 
+void gui_send_percent(gui_t *gui, int msgtype, int percent)
+{
+    mvprfbmsg_t **msglist;
+    mvprfbmsg_t  *msg;
+
+    Dprintf(DEBUG, "%s Sending message %d, %d\n", DEBUG_TIMESTAMP, msgtype, percent);
+
+    if ( initparams->dongle_version < 21365 ) {
+        msglist = rdc_msgs;
+    } else {
+        msglist = rdc_msgs_22012;
+    }
+
+    msg = msglist[msgtype];
+
+    msg->msg[19] = percent; // bad - modifies message directly
+
+    write(gui->fd,msg->msg,msg->len);
+}
 
 
-/** \brief Send the settings to the client down the RFB channel
+/** \internal
+ *  \brief Send the settings to the client down the RFB channel
  *
  *  \param gui Handle to the gui channel
  */
@@ -641,8 +697,8 @@ void gui_send_settings(gui_t *gui, int subcommand)
 
     gui_send_message(gui,RDC_SETTINGS);
 
-    buf[0] = subcommand;   
-    buf[1] = gui->dongle->tvmode;  
+    buf[0] = subcommand;
+    buf[1] = gui->dongle->tvmode;
     buf[2] = gui->dongle->videooutput;
     buf[3] = gui->dongle->flickermode;
     buf[4] = gui->dongle->aspectratio;
@@ -651,6 +707,9 @@ void gui_send_settings(gui_t *gui, int subcommand)
     write(gui->fd,buf,12);
 }
 
+
+
+
 /*
  * Local Variables:
  *  indent-tabs-mode:nil
