@@ -3,8 +3,8 @@
  *
  *   (C) 2003 Dominic Morris
  *
- *   $Id: msgs.h,v 1.2 2004/01/14 20:40:24 dom Exp $
- *   $Date: 2004/01/14 20:40:24 $
+ *   $Id: msgs.h,v 1.3 2004/03/16 20:07:05 dom Exp $
+ *   $Date: 2004/03/16 20:07:05 $
  *
  *
  *   Definitinons of various set rfb messages
@@ -37,8 +37,13 @@
 #define RDC_SETTINGS        0x0a
 #define RDC_DISPLAY_ON      0x0b          /* Fake */
 #define RDC_DISPLAY_OFF     0x0c          /* Fake */
-#define RFB_PONG            0x0d          /* Fake - internal */
-#define RDC_MESSAGE_MAX     0x0e          /* End marker */
+
+#define RDC_SKIP            0x0d          /* Fake */
+#define RDC_BACK            0x0e          /* Fake */
+#define RDC_SEEK_PERCENT    0x0f          /* Fake */
+
+#define RFB_PONG            0x10          /* Fake - internal */
+#define RDC_MESSAGE_MAX     0x11          /* End marker */
 
 #define RDC_SETTINGS_GET    0x00
 #define RDC_SETTINGS_TEST   0x01
